@@ -14,9 +14,9 @@ Puppet::Type.type(:mount).provide(:parsed,
     :filetype => :flat
 ) do
     include Puppet::Provider::Mount
-    confine :exists => fstab
+    #confine :exists => fstab
 
-    commands :mountcmd => "mount", :umount => "umount", :df => "df"
+    commands :mountcmd => "mount", :umount => "umount"
 
     @platform = Facter["operatingsystem"].value
     case @platform
@@ -34,4 +34,3 @@ Puppet::Type.type(:mount).provide(:parsed,
     record_line self.name, :fields => @fields, :separator => /\s+/, :joiner => "\t", :optional => [:pass, :dump]
 end
 
-# $Id: parsed.rb 2564 2007-06-11 22:49:06Z luke $
