@@ -1,213 +1,65 @@
 #!/usr/bin/env ruby
 
-$:.unshift("../../lib") if __FILE__ =~ /\.rb$/
+require File.dirname(__FILE__) + '/../../lib/puppettest'
 
 require 'puppettest'
 require 'mocha'
 
 class TestMasterClient < Test::Unit::TestCase
     include PuppetTest::ServerTest
-    
-    class FakeTrans
-        def initialize
-            @counters = Hash.new { |h,k| h[k] = 0 }
-        end
-        [:evaluate, :report, :cleanup, :addtimes, :tags, :ignoreschedules].each do |m|
-            define_method(m.to_s + "=") do |*args|
-                @counters[m] += 1
-            end
-            define_method(m) do |*args|
-                @counters[m] += 1
-            end
-            define_method(m.to_s + "?") do
-                @counters[m]
-            end
-        end
-    end
-    class FakeComponent
-        attr_accessor :trans
-        def evaluate
-            @trans = FakeTrans.new
-            @trans
-        end
-        
-        def finalize
-            @finalized = true
-        end
-        
-        def finalized?
-            @finalized
-        end
-    end
 
     def setup
         super
         @master = Puppet::Network::Client.master
     end
 
-    def mkmaster(file = nil)
-        master = nil
-
-        file ||= mktestmanifest()
+    def mkmaster(options = {})
+        options[:UseNodes] = false
+        options[:Local] = true
+        if code = options[:Code]
+            Puppet[:code] = code
+        else
+            Puppet[:manifest] = options[:Manifest] || mktestmanifest
+        end
         # create our master
-        assert_nothing_raised() {
-            # this is the default server setup
-            master = Puppet::Network::Handler.master.new(
-                :Manifest => file,
-                :UseNodes => false,
-                :Local => true
-            )
-        }
+        # this is the default server setup
+        master = Puppet::Network::Handler.master.new(options)
         return master
     end
 
     def mkclient(master = nil)
         master ||= mkmaster()
-        client = nil
-        assert_nothing_raised() {
-            client = Puppet::Network::Client.master.new(
-                :Master => master
-            )
-        }
+        client = Puppet::Network::Client.master.new(
+            :Master => master
+        )
 
         return client
     end
-    
-    def mk_fake_client
-        server = Puppet::Network::Handler.master.new :Code => ""
-        master = Puppet::Network::Client.master.new :Server => server, :Local => true
-
-        # Now create some objects
-        objects = FakeComponent.new
-
-        master.send(:instance_variable_set, "@objects", objects)
-
-        class << master
-            def report(r)
-                @reported ||= 0
-                @reported += 1
-            end
-            def reported
-                @reported ||= 0
-                @reported
-            end
-        end
-        return master, objects
-    end
-    
-    def test_apply
-        master, objects = mk_fake_client
-        
-        check = Proc.new do |hash|
-            assert(objects.trans, "transaction was not created")
-            trans = objects.trans
-            hash[:yes].each do |m|
-                assert_equal(1, trans.send(m.to_s + "?"), "did not call #{m} enough times")
-            end
-            hash[:no].each do |m|
-                assert_equal(0, trans.send(m.to_s + "?"), "called #{m} too many times")
-            end
-        end
-        
-        # First try it with no arguments
-        assert_nothing_raised do
-            master.apply
-        end
-        check.call :yes => %w{evaluate cleanup addtimes}, :no => %w{report tags ignoreschedules}
-        assert_equal(0, master.reported, "master sent report with reports disabled")
-        
-        
-        # Now enable reporting and make sure the report method gets called
-        Puppet[:report] = true
-        assert_nothing_raised do
-            master.apply
-        end
-        check.call :yes => %w{evaluate cleanup addtimes}, :no => %w{tags ignoreschedules}
-        assert_equal(1, master.reported, "master did not send report")
-        
-        # Now try it with tags enabled
-        assert_nothing_raised do
-            master.apply("tags")
-        end
-        check.call :yes => %w{evaluate cleanup tags addtimes}, :no => %w{ignoreschedules}
-        assert_equal(2, master.reported, "master did not send report")
-        
-        # and ignoreschedules
-        assert_nothing_raised do
-            master.apply("tags", true)
-        end
-        check.call :yes => %w{evaluate cleanup tags ignoreschedules addtimes}, :no => %w{}
-        assert_equal(3, master.reported, "master did not send report")
-    end
-    
-    def test_getconfig
-        client = mkclient
-        
-        $methodsrun = []
-        cleanup { $methodsrun = nil }
-        client.meta_def(:getplugins) do
-            $methodsrun << :getplugins
-        end
-        client.meta_def(:get_actual_config) do
-            $methodsrun << :get_actual_config
-            result = Puppet::TransBucket.new()
-            result.type = "testing"
-            result.name = "yayness"
-            result
-        end
-        
-        assert_nothing_raised do
-            client.getconfig
-        end
-        [:get_actual_config].each do |method|
-            assert($methodsrun.include?(method), "method %s was not run" % method)
-        end
-        assert(! $methodsrun.include?(:getplugins), "plugins were synced even tho disabled")
-
-        # Now set pluginsync
-        Puppet[:pluginsync] = true
-        $methodsrun.clear
-        
-        assert_nothing_raised do
-            client.getconfig
-        end
-        [:getplugins, :get_actual_config].each do |method|
-            assert($methodsrun.include?(method), "method %s was not run" % method)
-        end
-        
-        objects = client.objects
-        assert(objects.finalized?, "objects were not finalized")
-    end
 
     def test_disable
         FileUtils.mkdir_p(Puppet[:statedir])
         manifest = mktestmanifest
 
-        master = mkmaster(manifest)
+        master = mkmaster(:Manifest => manifest)
 
         client = mkclient(master)
 
-        assert(! FileTest.exists?(@createdfile))
-
-        assert_nothing_raised {
+        assert_nothing_raised("Could not disable client") {
             client.disable
         }
 
-        assert_nothing_raised {
-            client.run
-        }
+        client.expects(:getconfig).never
 
-        assert(! FileTest.exists?(@createdfile), "Disabled client ran")
+        client.run
 
-        assert_nothing_raised {
+        client = mkclient(master)
+
+        client.expects(:getconfig)
+
+        assert_nothing_raised("Could not enable client") {
             client.enable
         }
-
-        assert_nothing_raised {
-            client.run
-        }
-
-        assert(FileTest.exists?(@createdfile), "Enabled client did not run")
+        client.run
     end
 
     # Make sure we're getting the client version in our list of facts
@@ -240,7 +92,7 @@ class TestMasterClient < Test::Unit::TestCase
         }
     end
     
-    # This method is supposed
+    # This method downloads files, and yields each file object if a block is given.
     def test_download
         source = tempfile()
         dest = tempfile()
@@ -261,65 +113,41 @@ class TestMasterClient < Test::Unit::TestCase
     end
 
     def test_getplugins
+        Puppet[:filetimeout] = -1
         Puppet[:pluginsource] = tempfile()
         Dir.mkdir(Puppet[:pluginsource])
+        Dir.mkdir(File.join(Puppet[:pluginsource], "testing"))
 
-        myplugin = File.join(Puppet[:pluginsource], "myplugin.rb")
+        $loaded = []
+        loader = Puppet::Util::Autoload.new(self, "testing")
+
+        myplugin = File.join(Puppet[:pluginsource], "testing", "myplugin.rb")
         File.open(myplugin, "w") do |f|
-            f.puts %{Puppet::Type.newtype(:myplugin) do
-    newparam(:argument) do
-        isnamevar
-    end
-end
-}
+            f.puts %{$loaded << :myplugin}
         end
 
-        assert_nothing_raised {
+        assert_nothing_raised("Could not get plugins") {
             Puppet::Network::Client.master.getplugins
         }
 
-        destfile = File.join(Puppet[:plugindest], "myplugin.rb")
+        destfile = File.join(Puppet[:plugindest], "testing", "myplugin.rb")
 
         assert(File.exists?(destfile), "Did not get plugin")
 
-        obj = Puppet::Type.type(:myplugin)
+        assert(loader.load(:myplugin), "Did not load downloaded plugin")
 
-        assert(obj, "Did not define type")
-
-        assert(obj.validattr?(:argument),
-            "Did not get namevar")
+        assert($loaded.include?(:myplugin), "Downloaded code was not evaluated")
 
         # Now modify the file and make sure the type is replaced
         File.open(myplugin, "w") do |f|
-            f.puts %{Puppet::Type.newtype(:myplugin) do
-    newparam(:yayness) do
-        isnamevar
-    end
-
-    newparam(:rahness) do
-    end
-end
-}
+            f.puts %{$loaded << :changed}
         end
 
-        assert_nothing_raised {
+        assert_nothing_raised("Could not get plugin changes") {
             Puppet::Network::Client.master.getplugins
         }
 
-        destfile = File.join(Puppet[:pluginpath], "myplugin.rb")
-
-        obj = Puppet::Type.type(:myplugin)
-
-        assert(obj, "Did not define type")
-
-        assert(obj.validattr?(:yayness),
-            "Did not get namevar")
-
-        assert(obj.validattr?(:rahness),
-            "Did not get other var")
-
-        assert(! obj.validattr?(:argument),
-            "Old namevar is still valid")
+        assert($loaded.include?(:changed), "Changed code was not evaluated")
 
         # Now try it again, to make sure we don't have any objects lying around
         assert_nothing_raised {
@@ -328,6 +156,7 @@ end
     end
 
     def test_getfacts
+        Puppet[:filetimeout] = -1
         Puppet[:factsource] = tempfile()
         Dir.mkdir(Puppet[:factsource])
         hostname = Facter.value(:hostname)
@@ -350,6 +179,7 @@ end
 
         assert_equal(hostname, Facter.value(:hostname),
             "Lost value to hostname")
+
         assert_equal("yayness", Facter.value(:myfact),
             "Did not get correct fact value")
 
@@ -381,6 +211,16 @@ end
             "Lost value to hostname")
     end
 
+    # Make sure that setting environment by fact takes precedence to configuration
+    def test_setenvironmentwithfact
+        name = "environment"
+        value = "test_environment"
+
+        Facter.stubs(:to_hash).returns(name => value)
+
+        assert_equal(value, Puppet::Network::Client.master.facts[name])
+    end
+
     # Make sure we load all facts on startup.
     def test_loadfacts
         dirs = [tempfile(), tempfile()]
@@ -455,7 +295,7 @@ end
             facts = Puppet::Network::Client.master.facts
         end
         Facter.to_hash.each do |fact, value|
-            assert_equal(facts[fact.downcase], value, "%s is not equal" % fact.inspect)
+            assert_equal(facts[fact.downcase], value.to_s, "%s is not equal" % fact.inspect)
         end
         
         # Make sure the puppet version got added
@@ -471,7 +311,9 @@ end
         manifest = tempfile()
         File.open(manifest, "w") { |f| f.puts "file { '#{file}': content => yay }" }
         
-        driver = mkmaster(manifest)
+        Puppet::Node::Facts.indirection.stubs(:save)
+        
+        driver = mkmaster(:Manifest => manifest)
         driver.local = false
         master = mkclient(driver)
         
@@ -480,7 +322,7 @@ end
         
         assert(! master.fresh?(master.class.facts),
             "Considered fresh with no compile at all")
-        
+
         assert_nothing_raised { master.run }
         assert(master.fresh?(master.class.facts),
             "not considered fresh after compile")
@@ -501,45 +343,14 @@ end
         assert(FileTest.exists?(file), "file was not created on second run")
     end
 
-    def test_default_objects
-        # Make sure they start out missing
-        assert_nil(Puppet::Type.type(:filebucket)["puppet"],
-            "default filebucket already exists")
-        assert_nil(Puppet::Type.type(:schedule)["daily"],
-            "default schedules already exists")
-
-        master = mkclient()
-
-        # Now make sure they got created
-        assert(Puppet::Type.type(:filebucket)["puppet"],
-            "default filebucket not found")
-        assert(Puppet::Type.type(:schedule)["daily"],
-            "default schedules not found")
-
-        # clear everything, and make sure we can recreate them
-        Puppet::Type.allclear
-        assert_nil(Puppet::Type.type(:filebucket)["puppet"],
-            "default filebucket not removed")
-        assert_nil(Puppet::Type.type(:schedule)["daily"],
-            "default schedules not removed")
-        assert_nothing_raised  { master.mkdefault_objects }
-        assert(Puppet::Type.type(:filebucket)["puppet"],
-            "default filebucket not found")
-        assert(Puppet::Type.type(:schedule)["daily"],
-            "default schedules not found")
-
-
-        # Make sure we've got schedules
-        assert(Puppet::Type.type(:schedule)["hourly"], "Could not retrieve hourly schedule")
-        assert(Puppet::Type.type(:filebucket)["puppet"], "Could not retrieve default bucket")
-    end
-
     # #540 - make sure downloads aren't affected by noop
     def test_download_in_noop
         source = tempfile
         File.open(source, "w") { |f| f.puts "something" }
         dest = tempfile
         Puppet[:noop] = true
+        node = stub 'node', :environment => "development"
+        Puppet::Node.stubs(:find).returns node
         assert_nothing_raised("Could not download in noop") do
             @master.download(:dest => dest, :source => source, :tag => "yay")
         end
@@ -555,7 +366,9 @@ end
         master.local = false
         driver = master.send(:instance_variable_get, "@driver")
         driver.local = false
+        Puppet::Node::Facts.indirection.stubs(:save)
         # Retrieve the configuration
+
         master.getconfig
 
         # Now the config is up to date, so get rid of the @objects var and
@@ -583,6 +396,8 @@ end
         driver = master.send(:instance_variable_get, "@driver")
         driver.local = false
 
+        Puppet::Node::Facts.indirection.stubs(:save)
+
         assert_nothing_raised("Could not compile config") do
             master.getconfig
         end
@@ -616,9 +431,7 @@ end
             end
         end
 
-        assert_raise(ArgumentError, "did not fail") do
-            master.run
-        end
+        master.run
 
         assert(! master.send(:lockfile).locked?,
             "Master is still locked after failure")
@@ -707,22 +520,89 @@ end
         end
     end
 
+    def test_environment_is_added_to_facts
+        facts = Puppet::Network::Client::Master.facts
+        assert_equal(facts["environment"], Puppet[:environment], "Did not add environment to client facts")
+
+        # Now set it to a real value
+        Puppet[:environments] = "something,else"
+        Puppet[:environment] = "something"
+        facts = Puppet::Network::Client::Master.facts
+        assert_equal(facts["environment"], Puppet[:environment], "Did not add environment to client facts")
+    end
+
     # This is partially to fix #532, but also to save on memory.
     def test_remove_objects_after_every_run
         client = mkclient
 
         ftype = Puppet::Type.type(:file)
+        file = ftype.create :title => "/what/ever", :ensure => :present
+        config = Puppet::Node::Catalog.new
+        config.add_resource(file)
 
-        assert_nil(ftype[@createdfile], "file object already exists")
-        assert(! FileTest.exists?(@createdfile), "File already exists on disk")
+        config.expects :apply
 
-        assert_nothing_raised("Could not apply config") do
-            client.run
-        end
+        client.catalog = config
+        client.expects(:getconfig)
+        client.run
 
-        assert(FileTest.exists?(@createdfile), "File does not exist on disk")
         assert_nil(ftype[@createdfile], "file object was not removed from memory")
     end
-end
 
-# $Id: master.rb 2545 2007-05-31 22:47:01Z luke $
+    # #685
+    def test_http_failures_do_not_kill_puppetd
+        client = mkclient
+
+        client.meta_def(:getconfig) { raise "A failure" }
+
+        assert_nothing_raised("Failure in getconfig threw an error") do
+            client.run
+        end
+    end
+
+    def test_invalid_catalogs_do_not_get_cached
+        master = mkmaster :Code => "notify { one: require => File[yaytest] }"
+        master.local = false # so it gets cached
+        client = mkclient(master)
+        client.stubs(:facts).returns({})
+        client.local = false
+
+        Puppet::Node::Facts.indirection.stubs(:terminus_class).returns(:memory)
+
+        # Make sure the config is not cached.
+        client.expects(:cache).never
+
+        client.getconfig
+        # Doesn't throw an exception, but definitely fails.
+        client.run
+    end
+
+    def test_classfile
+        Puppet[:code] = "class yaytest {}\n class bootest {}\n include yaytest, bootest"
+
+        Puppet::Node::Facts.indirection.stubs(:save)
+
+        master = client = nil
+        assert_nothing_raised() {
+            master = Puppet::Network::Handler.master.new(
+                :Local => false
+            )
+        }
+        assert_nothing_raised() {
+            client = Puppet::Network::Client.master.new(
+                :Master => master
+            )
+        }
+
+        # Fake that it's local, so it creates the class file
+        client.local = false
+
+        # We can't guarantee class ordering
+        client.expects(:setclasses).with do |array|
+            array.length == 2 and array.include?("yaytest") and array.include?("bootest")
+        end
+        assert_nothing_raised {
+            client.getconfig
+        }
+    end
+end
