@@ -3,13 +3,13 @@
 #  Created by Luke A. Kanies on 2006-12-21.
 #  Copyright (c) 2006. All rights reserved.
 
-$:.unshift("../lib").unshift("../../lib") if __FILE__ =~ /\.rb$/
+require File.dirname(__FILE__) + '/../lib/puppettest'
 
 require 'puppettest'
 
 class TestPropertyChange < Test::Unit::TestCase
 	include PuppetTest
-	class FakeProperty < Puppet::Type::Property
+	class FakeProperty < Puppet::Property
 	    attr_accessor :is, :should, :resource
 	    attr_reader :noop
 	    def change_to_s(currentvalue, newvalue)
@@ -138,4 +138,3 @@ class TestPropertyChange < Test::Unit::TestCase
     end
 end
 
-# $Id: propertychange.rb 2517 2007-05-15 19:22:36Z luke $
